@@ -2,11 +2,14 @@
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelWorkflow, ModelSQL, fields
 from trytond.wizard import Wizard
+from trytond.pyson import Not, Equal, Eval, Or, Bool
+from trytond.backend import TableHandler
 import datetime
-import mx.DateTime
+import time
+from dateutil.relativedelta import relativedelta
 
 STATES = {
-    'readonly': "state != 'draft'",
+    'readonly': Not(Equal(Eval('state'), 'draft')),
 }
 
 
@@ -19,7 +22,8 @@ class Forecast(ModelWorkflow, ModelSQL, ModelView):
     location = fields.Many2One(
         'stock.location', 'Location', required=True,
         domain=[('type', '=', 'storage')], states={
-            'readonly': "state != 'draft' or bool(lines)",
+            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                Bool(Eval('lines'))),
         })
     destination = fields.Many2One(
         'stock.location', 'Destination', required=True,
@@ -30,7 +34,8 @@ class Forecast(ModelWorkflow, ModelSQL, ModelView):
         'stock.forecast.line', 'forecast', 'Lines', states=STATES)
     company = fields.Many2One(
         'company.company', 'Company', required=True, states={
-            'readonly': "state != 'draft' or bool(lines)",
+            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                Bool(Eval('lines'))),
         })
     state = fields.Selection([
         ('draft', 'Draft'),
@@ -58,6 +63,13 @@ class Forecast(ModelWorkflow, ModelSQL, ModelView):
         self._order.insert(0, ('from_date', 'DESC'))
         self._order.insert(1, ('location', 'ASC'))
 
+    def init(self, cursor, module_name):
+        super(Forecast, self).init(cursor, module_name)
+
+        # Add index on create_date
+        table = TableHandler(cursor, self, module_name)
+        table.index_action('create_date', action='add')
+
     def default_state(self, cursor, user, context=None):
         return 'draft'
 
@@ -70,7 +82,6 @@ class Forecast(ModelWorkflow, ModelSQL, ModelView):
         return False
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
@@ -166,12 +177,16 @@ class ForecastLine(ModelSQL, ModelView):
             domain=[('type', '=', 'stockable')], on_change=['product'])
     uom = fields.Many2One(
         'product.uom', 'UOM', required=True,
-        domain=["('category', '=', (product, 'product.default_uom.category'))"])
-    unit_digits = fields.Function('get_unit_digits', type='integer',
-            string='Unit Digits')
-    quantity = fields.Float('Quantity', digits="(16, unit_digits)", required=True)
-    minimal_quantity = fields.Float(
-        'Minimal Qty', digits="(16, unit_digits)", required=True)
+        domain=[
+            ('category', '=',
+                (Eval('product'), 'product.default_uom.category')),
+        ], on_change=['uom'])
+    unit_digits = fields.Function(fields.Integer('Unit Digits'),
+            'get_unit_digits')
+    quantity = fields.Float('Quantity', digits=(16, Eval('unit_digits', 2)),
+            required=True)
+    minimal_quantity = fields.Float('Minimal Qty',
+            digits=(16, Eval('unit_digits', 2)), required=True)
     moves = fields.Many2Many('stock.forecast.line-stock.move',
             'line', 'move','Moves', readonly=True)
     forecast = fields.Many2One(
@@ -195,9 +210,8 @@ class ForecastLine(ModelSQL, ModelView):
     def default_minimal_quantity(self, cursor, user, context=None):
         return 1.0
 
-    def on_change_product(self, cursor, user, ids, vals, context=None):
+    def on_change_product(self, cursor, user, vals, context=None):
         product_obj = self.pool.get('product.product')
-        uom_obj = self.pool.get('product.uom')
         res = {}
         res['unit_digits'] = 2
         if vals.get('product'):
@@ -208,7 +222,16 @@ class ForecastLine(ModelSQL, ModelView):
             res['unit_digits'] = product.default_uom.digits
         return res
 
-    def get_unit_digits(self, cursor, user, ids, name, arg, context=None):
+    def on_change_uom(self, cursor, user, vals, context=None):
+        uom_obj = self.pool.get('product.uom')
+        res = {}
+        res['unit_digits'] = 2
+        if vals.get('uom'):
+            uom = uom_obj.browse(cursor, user, vals['uom'], context=context)
+            res['unit_digits'] = uom.digits
+        return res
+
+    def get_unit_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
             res[line.id] = line.product.default_uom.digits
@@ -380,10 +403,7 @@ class ForecastComplete(Wizard):
 
         res = {}
         for field in ("to_date", "from_date"):
-            date = mx.DateTime.strptime(str(forecast[field]), '%Y-%m-%d')
-            new_date = date - mx.DateTime.RelativeDateTime(years=1)
-            res[field] = datetime.date(new_date.year, new_date.month,
-                    new_date.day)
+            res[field] = forecast[field] - relativedelta(years=1)
         return res
 
     def _get_product_quantity(self, cursor, user, data, context=None):
